#include "csapp.h"

#define TICK_EVERY 3

int main(int argc, char **argv) {
  char *portno;
  struct addrinfo hints, *addrs;
  int s;

  if (argc != 2)
    app_error("need <port>");

  portno = argv[1];

  memset(&hints, 0, sizeof(struct addrinfo));
  hints.ai_family = AF_INET;        /* Request IPv4 */
  hints.ai_socktype = SOCK_DGRAM;   /* Accept UDP connections */
  hints.ai_flags = AI_PASSIVE;      /* ... on any IP address */
  Getaddrinfo(NULL, portno, &hints, &addrs);

  s = Socket(addrs->ai_family, addrs->ai_socktype, addrs->ai_protocol);
  Bind(s, addrs->ai_addr, addrs->ai_addrlen);
  Freeaddrinfo(addrs);

  printf("Ticking at %d-second boundaries\n", TICK_EVERY);
  while (1) {
    char buffer[MAXBUF];
    struct timeval now;
    long now_s;
    struct timespec delay;
    struct sockaddr_in from_addr;
    unsigned int from_len = sizeof(from_addr);

    Recvfrom(s, buffer, MAXBUF, 0,
	     (struct sockaddr *)&from_addr, &from_len);

    gettimeofday(&now, NULL);
    if ((now.tv_usec == 0)
	&& ((now.tv_sec % TICK_EVERY) == 0)) {
      delay.tv_sec = TICK_EVERY;
      delay.tv_nsec = 0;
    } else {
      delay.tv_sec = (TICK_EVERY - 1 - (now.tv_sec % TICK_EVERY));
      delay.tv_nsec = (1000000 - now.tv_usec) * 1000;
    }

    while (nanosleep(&delay, &delay)) { }

    gettimeofday(&now, NULL);
    now_s = now.tv_sec;
    printf("tick %ld\n", now_s);
    
    Sendto(s, &now_s, sizeof(long), 0, (SA*)&from_addr, from_len);
  }
}
